from __future__ import annotations

import shutil
import subprocess
from collections.abc import Iterable
from enum import Enum
from pathlib import Path
from typing import NamedTuple

from pydantic import BaseModel
from rich.console import Console
from rich.markup import escape


class ParameterKind(str, Enum):
    required = "required"
    optional = "optional"
    variadic = "variadic"


class Scope(str, Enum):
    global_ = "global"
    local = "local"


class WorkingDirMode(str, Enum):
    cwd = "cwd"
    justfile = "justfile"


class RecipeDefault(BaseModel):
    """The default value of a recipe parameter.

    Attributes:
        value: Raw value from just --dump (string, list, or None for empty string default).
        expression: True if the default is a complex expression (list), False for string literals.
    """

    value: str | list | None
    expression: bool


class Parameter(BaseModel):
    """A parameter declared in a just recipe.

    Attributes:
        name: Parameter name.
        default: Default value, or None if not set.
        kind: Whether the parameter is required, optional, or variadic.
    """

    name: str
    default: RecipeDefault | None
    kind: ParameterKind


class Recipe(BaseModel):
    """A just recipe (target).

    Attributes:
        name: Recipe name.
        doc: Doc comment from the justfile, or None.
        parameters: Ordered list of parameters.
        dependencies: Names of recipes this recipe depends on.
    """

    name: str
    doc: str | None
    parameters: list[Parameter]
    dependencies: list[str]
    groups: list[str] = []


class RecipeGroup(NamedTuple):
    name: str | None
    recipes: list[Recipe]


class Source(BaseModel):
    """A justfile source (global or local).

    Attributes:
        name: Display name for the source.
        scope: Whether this is a global or local justfile.
        path: Absolute path to the justfile.
        recipes: Recipes defined in this justfile.
    """

    name: str
    scope: Scope
    path: Path
    recipes: list[Recipe]
    working_dir_mode: WorkingDirMode = WorkingDirMode.cwd

    def filter_recipes(self, query: str = "") -> list[Recipe]:
        """Return visible recipes matching query (case-insensitive substring on name, doc, groups, source name)."""
        visible = [r for r in self.recipes if not r.name.startswith("_")]
        if not query:
            return visible
        q = query.lower()
        return [
            r
            for r in visible
            if q in r.name.lower()
            or (r.doc is not None and q in r.doc.lower())
            or any(q in g.lower() for g in r.groups)
            or q in self.name.lower()
        ]

    def run(self, recipe_name: str, args: Iterable[str] = ()) -> int:
        from justx.justfiles.exceptions import JustNotFoundError

        just_bin = shutil.which("just")
        if just_bin is None:
            raise JustNotFoundError()
        working_directory = self.path.parent if self.working_dir_mode == WorkingDirMode.justfile else Path.cwd()
        result = subprocess.run(
            [just_bin, "--justfile", str(self.path), "--working-directory", str(working_directory), recipe_name, *args],
            check=False,
        )
        return result.returncode

    def pretty_print(self, console: Console | None = None) -> None:
        console = console or Console()
        scope_label = self.scope.value
        console.print(
            f"[bold]{escape(self.name)}[/bold]  [dim]\\[{scope_label}][/dim]  [dim]{escape(str(self.path))}[/dim]"
        )
        for recipe in self.recipes:
            params = " ".join(p.name for p in recipe.parameters)
            signature = escape(f"{recipe.name} {params}".strip())
            doc = f"  [dim]# {escape(recipe.doc)}[/dim]" if recipe.doc else ""
            console.print(f"  [cyan]{signature}[/cyan]{doc}")
        console.print()


class JustxConfig(BaseModel):
    global_sources: list[Source]
    local_sources: list[Source]



from __future__ import annotations

from importlib.metadata import version

import click

from justx.cli.commands import check_cmd, init_cmd, list_cmd, run_cmd
from justx.justfiles.loader import JustxLoader
from justx.tui import run_tui


def display_version(ctx: click.Context, _param: click.Parameter, value: bool) -> None:
    if not value or ctx.resilient_parsing:
        return
    click.echo(f"justx {version('justx')}")
    ctx.exit()


@click.group(invoke_without_command=True)
@click.option(
    "--version",
    is_flag=True,
    is_eager=True,
    expose_value=False,
    callback=display_version,
    help="Show version and exit.",
)
@click.pass_context
def main(ctx: click.Context) -> None:
    """justx — a TUI launcher for just recipes."""
    if ctx.invoked_subcommand is None:
        config = JustxLoader().load()
        run_tui(config)


main.add_command(list_cmd)
main.add_command(run_cmd)
main.add_command(check_cmd)
main.add_command(init_cmd)



from justx.cli.commands.check import check_cmd
from justx.cli.commands.init import init_cmd
from justx.cli.commands.list import list_cmd
from justx.cli.commands.run import run_cmd

__all__ = ["check_cmd", "init_cmd", "list_cmd", "run_cmd"]



from pathlib import Path

import pytest

from justx.justfiles.models import ParameterKind, Scope
from justx.justfiles.parser import JustfileParser


def test_parse_justfile(example_justfile):
    source = JustfileParser().parse(example_justfile, Scope.local)

    assert source.name == "justfile"
    assert source.path == example_justfile

    recipes = {r.name: r for r in source.recipes}
    assert set(recipes) == {"bootstrap", "upgrade-deps", "script1", "script2"}

    assert recipes["bootstrap"].doc == "Set up development environment"
    assert recipes["bootstrap"].parameters == []
    assert recipes["bootstrap"].dependencies == []

    assert recipes["upgrade-deps"].dependencies == ["bootstrap"]

    assert recipes["script1"].parameters == []

    args = recipes["script2"].parameters[0]
    assert args.name == "ARGS"
    assert args.kind == ParameterKind.variadic
    assert args.default is None


def test_parse_justfile_not_found():
    with pytest.raises(FileNotFoundError):
        JustfileParser().parse(Path("nonexistent/justfile"), Scope.local)



from __future__ import annotations

from pathlib import Path

from click.testing import CliRunner

from justx.cli.main import main
from tests.utils import run_within_dir


def test_run_no_scope(local_dir: Path) -> None:
    runner = CliRunner()
    with run_within_dir(local_dir):
        result = runner.invoke(main, ["run", "simple:greet"])
    assert result.exit_code == 2
    assert "Specify scope with -l (local) or -g (global)." in result.output


def test_run_conflicting_scope(local_dir: Path) -> None:
    runner = CliRunner()
    with run_within_dir(local_dir):
        result = runner.invoke(main, ["run", "-g", "-l", "simple:greet"])
    assert result.exit_code == 2
    assert "Cannot use -g and -l together." in result.output


def test_run_local_recipe(local_dir: Path) -> None:
    runner = CliRunner()
    with run_within_dir(local_dir):
        result = runner.invoke(main, ["run", "-l", "simple:greet"])
    assert result.exit_code == 0


def test_run_global_recipe(global_dir: Path) -> None:
    runner = CliRunner()
    with run_within_dir(global_dir):
        result = runner.invoke(main, ["run", "-g", "setup:setup"])
    assert result.exit_code == 0


def test_run_local_source_not_found(tmp_path: Path) -> None:
    runner = CliRunner()
    with run_within_dir(tmp_path):
        result = runner.invoke(main, ["run", "-l", "greet"])
    assert result.exit_code == 1
    assert "not found in local sources" in result.output


def test_run_global_source_not_found(tmp_path: Path) -> None:
    runner = CliRunner()
    with run_within_dir(tmp_path):
        result = runner.invoke(main, ["run", "-g", "greet"], env={"JUSTX_HOME": str(tmp_path)})
    assert result.exit_code == 1
    assert "not found in global sources" in result.output



from __future__ import annotations

from justx.justfiles.models import Source


def parse_target(target: str) -> tuple[str | None, str]:
    """Split a ``source:recipe`` target string into its components.

    When no colon is present the target is treated as a plain recipe on the root justfile.

    Args:
        target: A raw CLI argument such as ``"docker:build"`` or ``"build"``.
    """
    if ":" in target:
        source_name, recipe = target.split(":", 1)
        return source_name, recipe
    return None, target


def find_source(sources: list[Source], source_name: str | None) -> Source | None:
    """Look up a source by name, or return the root justfile.

    Sources are identified by their file name.  The root justfile has no
    source name, so we match on the conventional ``justfile`` / ``Justfile``
    filenames rather than requiring callers to know the exact casing.

    Args:
        sources: The list of sources to search (local or global).
        source_name: The source name to match, or ``None`` to find the root justfile.
    """
    for source in sources:
        if source_name is None and source.name in ("justfile", "Justfile"):
            return source
        if source_name is not None and source.name == source_name:
            return source
    return None



from __future__ import annotations

from typing import ClassVar, NamedTuple

from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import Horizontal
from textual.message import Message
from textual.screen import Screen
from textual.widgets import Footer, Header, Input

from justx.justfiles.models import JustxConfig, Recipe, Source
from justx.tui.screens.recipe import RecipeScreen
from justx.tui.screens.recipe_detail import RecipeDetailScreen
from justx.tui.widgets import RecipesPane, SourcesPane


class Selection(NamedTuple):
    source: Source
    recipe: Recipe
    args: list[str]


class SearchInput(Input):
    class Submitted(Message):
        """Posted when the user confirms (Enter or Down)."""

    class Cancelled(Message):
        """Posted when the user cancels (Escape). Input is cleared before posting."""

    BINDINGS: ClassVar = [
        Binding("enter", "focus_sources", "Confirm"),
        Binding("down", "focus_sources", show=False),
        Binding("escape", "clear_and_focus", "Cancel"),
    ]

    def action_focus_sources(self) -> None:
        self.post_message(self.Submitted())

    def action_clear_and_focus(self) -> None:
        self.clear()
        self.post_message(self.Cancelled())


class RecipeSelectionScreen(Screen[Selection | None]):
    CSS = """
    #search-input {
        dock: top;
        border: solid dodgerblue 40%;
    }
    #search-input:focus {
        border: solid dodgerblue;
    }
    Horizontal {
        height: 1fr;
    }
    #sources {
        width: 1fr;
        margin-right: 1;
        border: solid dodgerblue 40%;
        border-title-color: dodgerblue 40%;
    }
    #sources:focus-within {
        border: solid dodgerblue;
        border-title-color: dodgerblue;
    }
    #recipes {
        width: 2fr;
        border: solid dodgerblue 40%;
        border-title-color: dodgerblue 40%;
    }
    #recipes:focus-within {
        border: solid dodgerblue;
        border-title-color: dodgerblue;
    }
    """

    BINDINGS: ClassVar = [
        Binding("left", "focus_sources", "Sources"),
        Binding("right", "focus_recipes", "Recipes"),
        Binding("s", "focus_search", "Search", show=True),
    ]

    def __init__(self, config: JustxConfig) -> None:
        super().__init__()
        self._config = config
        self._selected_source: Source | None = None

    def compose(self) -> ComposeResult:
        yield Header()
        yield SearchInput(placeholder="Search recipes...", id="search-input")
        with Horizontal():
            yield SourcesPane(self._config)
            yield RecipesPane()
        yield Footer()

    def on_mount(self) -> None:
        self.query_one(SourcesPane).focus()

    def on_input_changed(self, event: Input.Changed) -> None:
        query = event.value.strip()
        sources_pane = self.query_one(SourcesPane)
        recipes_pane = self.query_one(RecipesPane)
        sources_pane.filter(query)
        recipes_pane.filter(query)

    def on_sources_pane_source_selected(self, message: SourcesPane.SourceSelected) -> None:
        self._selected_source = message.source
        self.query_one(RecipesPane).set_source(message.source)

    def on_sources_pane_source_activated(self, message: SourcesPane.SourceActivated) -> None:
        self.action_focus_recipes()

    def on_recipes_pane_recipe_run(self, message: RecipesPane.RecipeRun) -> None:
        recipe = message.recipe
        source = self._selected_source
        if not recipe.parameters:
            self.dismiss(Selection(source=source, recipe=recipe, args=[]))
        else:
            self.app.push_screen(
                RecipeScreen(recipe, source),
                lambda args: (
                    self.app.call_later(self.dismiss, Selection(source=source, recipe=recipe, args=args))
                    if args is not None
                    else None
                ),
            )

    def on_recipes_pane_recipe_details(self, message: RecipesPane.RecipeDetails) -> None:
        self.app.push_screen(RecipeDetailScreen(message.recipe, self._selected_source))

    def on_search_input_submitted(self, message: SearchInput.Submitted) -> None:
        self.action_focus_sources()

    def on_search_input_cancelled(self, message: SearchInput.Cancelled) -> None:
        self.action_focus_sources()

    def action_dismiss_screen(self) -> None:
        self.dismiss(None)

    def action_focus_sources(self) -> None:
        self.query_one(SourcesPane).focus()

    def action_focus_recipes(self) -> None:
        self.query_one(RecipesPane).focus_first_enabled()

    def action_focus_search(self) -> None:
        self.query_one("#search-input", Input).focus()



from __future__ import annotations

from pathlib import Path

import pytest

from justx.config import DEFAULT_JUSTX_HOME
from justx.justfiles.discovery import DiscoveredPaths, JustxDiscovery


def _write_justfile(path: Path, content: str = "hello:\n    echo hello\n") -> Path:
    path.parent.mkdir(parents=True, exist_ok=True)
    path.write_text(content)
    return path


@pytest.fixture
def tmp_home(tmp_path: Path) -> Path:
    return tmp_path / "justx_home"


@pytest.fixture
def tmp_cwd(tmp_path: Path) -> Path:
    return tmp_path / "project"


def test_discover_empty(tmp_home, tmp_cwd):
    tmp_cwd.mkdir()
    result = JustxDiscovery().discover(cwd=tmp_cwd, justx_home=tmp_home)
    assert result.global_paths == []
    assert result.local_paths == []


def test_discover_global_root_justfile(tmp_home, tmp_cwd):
    tmp_cwd.mkdir()
    _write_justfile(tmp_home / "justfile")

    result = JustxDiscovery().discover(cwd=tmp_cwd, justx_home=tmp_home)

    assert result.global_paths == [tmp_home / "justfile"]
    assert result.local_paths == []


def test_discover_local_root_justfile(tmp_home, tmp_cwd):
    tmp_cwd.mkdir()
    _write_justfile(tmp_cwd / "justfile")

    result = JustxDiscovery().discover(cwd=tmp_cwd, justx_home=tmp_home)

    assert result.global_paths == []
    assert result.local_paths == [tmp_cwd / "justfile"]


def test_discover_global_just_files(tmp_home, tmp_cwd):
    tmp_cwd.mkdir()
    _write_justfile(tmp_home / "ops.just")
    _write_justfile(tmp_home / "dev.just")

    result = JustxDiscovery().discover(cwd=tmp_cwd, justx_home=tmp_home)

    assert set(result.global_paths) == {tmp_home / "ops.just", tmp_home / "dev.just"}
    assert result.local_paths == []


def test_discover_local_just_files(tmp_home, tmp_cwd):
    tmp_cwd.mkdir()
    _write_justfile(tmp_cwd / ".justx" / "ci.just")

    result = JustxDiscovery().discover(cwd=tmp_cwd, justx_home=tmp_home)

    assert result.global_paths == []
    assert result.local_paths == [tmp_cwd / ".justx" / "ci.just"]


def test_discover_global_root_comes_before_just_files(tmp_home, tmp_cwd):
    tmp_cwd.mkdir()
    _write_justfile(tmp_home / "justfile")
    _write_justfile(tmp_home / "ops.just")

    result = JustxDiscovery().discover(cwd=tmp_cwd, justx_home=tmp_home)

    assert result.global_paths[0] == tmp_home / "justfile"
    assert result.global_paths[1] == tmp_home / "ops.just"


def test_discover_local_root_comes_before_just_files(tmp_home, tmp_cwd):
    tmp_cwd.mkdir()
    _write_justfile(tmp_cwd / "justfile")
    _write_justfile(tmp_cwd / ".justx" / "ci.just")

    result = JustxDiscovery().discover(cwd=tmp_cwd, justx_home=tmp_home)

    assert result.local_paths[0] == tmp_cwd / "justfile"
    assert result.local_paths[1] == tmp_cwd / ".justx" / "ci.just"


def test_justx_home_env_var(monkeypatch, tmp_path):
    home = tmp_path / "custom_home"
    _write_justfile(home / "justfile")
    cwd = tmp_path / "cwd"
    cwd.mkdir()

    monkeypatch.setenv("JUSTX_HOME", str(home))
    result = JustxDiscovery().discover(cwd=cwd)

    assert result.global_paths == [home / "justfile"]


def test_justx_home_arg_takes_precedence_over_env_var(monkeypatch, tmp_path):
    env_home = tmp_path / "env_home"
    _write_justfile(env_home / "justfile")

    arg_home = tmp_path / "arg_home"
    _write_justfile(arg_home / "dev.just")

    cwd = tmp_path / "cwd"
    cwd.mkdir()

    monkeypatch.setenv("JUSTX_HOME", str(env_home))
    result = JustxDiscovery().discover(cwd=cwd, justx_home=arg_home)

    assert result.global_paths == [arg_home / "dev.just"]


def test_discover_global_justfile_extension(tmp_home, tmp_cwd):
    tmp_cwd.mkdir()
    _write_justfile(tmp_home / "ops.justfile")
    _write_justfile(tmp_home / "dev.justfile")

    result = JustxDiscovery().discover(cwd=tmp_cwd, justx_home=tmp_home)

    assert set(result.global_paths) == {tmp_home / "ops.justfile", tmp_home / "dev.justfile"}
    assert result.local_paths == []


def test_discover_local_justfile_extension(tmp_home, tmp_cwd):
    tmp_cwd.mkdir()
    _write_justfile(tmp_cwd / ".justx" / "ci.justfile")

    result = JustxDiscovery().discover(cwd=tmp_cwd, justx_home=tmp_home)

    assert result.global_paths == []
    assert result.local_paths == [tmp_cwd / ".justx" / "ci.justfile"]


def test_default_home_is_dot_justx():
    assert Path.home() / ".justx" == DEFAULT_JUSTX_HOME


def test_returns_discovered_paths_instance(tmp_home, tmp_cwd):
    tmp_cwd.mkdir()
    result = JustxDiscovery().discover(cwd=tmp_cwd, justx_home=tmp_home)
    assert isinstance(result, DiscoveredPaths)



from justx.tui.utils import first_enabled_index
from justx.tui.widgets.recipes_pane import RecipesPane
from justx.tui.widgets.sources_pane import SourcesPane

__all__ = ["RecipesPane", "SourcesPane", "first_enabled_index"]



from __future__ import annotations

from pathlib import Path

from click.testing import CliRunner

from justx.cli.main import main
from tests.utils import run_within_dir


def test_list_no_sources(tmp_path: Path) -> None:
    runner = CliRunner()
    with run_within_dir(tmp_path):
        result = runner.invoke(main, ["list"], env={"JUSTX_HOME": str(tmp_path)})
    assert result.exit_code == 0
    assert "No justfiles found." in result.output


def test_list_both_scopes(local_dir: Path, global_dir: Path) -> None:
    runner = CliRunner()
    with run_within_dir(local_dir):
        result = runner.invoke(main, ["list"])
    assert result.exit_code == 0
    assert "greet" in result.output  # from local simple.just
    assert "setup" in result.output  # from global justfile
    assert "Greet someone" in result.output
    assert "Global setup task" in result.output


def test_list_global_only(local_dir: Path) -> None:
    runner = CliRunner()
    with run_within_dir(local_dir):
        result = runner.invoke(main, ["list", "-g"])
    assert result.exit_code == 0
    assert "setup" in result.output
    assert "Global setup task" in result.output
    assert "greet" not in result.output


def test_list_local_only(local_dir: Path) -> None:
    runner = CliRunner()
    with run_within_dir(local_dir):
        result = runner.invoke(main, ["list", "-l"])
    assert result.exit_code == 0
    assert "greet" in result.output
    assert "Greet someone" in result.output
    assert "Global setup task" not in result.output


def test_list_conflicting_flags(local_dir: Path) -> None:
    runner = CliRunner()
    with run_within_dir(local_dir):
        result = runner.invoke(main, ["list", "-g", "-l"])
    assert result.exit_code == 2
    assert "Cannot use -g and -l together." in result.output


def test_list_with_source_filter(local_dir: Path) -> None:
    runner = CliRunner()
    with run_within_dir(local_dir):
        result = runner.invoke(main, ["list", "-g", "setup"])
    assert result.exit_code == 0
    assert "setup" in result.output
    assert "Global setup task" in result.output
    assert "greet" not in result.output



from __future__ import annotations

from typing import Any, ClassVar

from textual.binding import Binding
from textual.containers import Vertical
from textual.message import Message
from textual.widgets import Label, ListItem, ListView

from justx.justfiles.models import Recipe, Source
from justx.justfiles.utils import group_recipes
from justx.tui.utils import first_enabled_index


class RecipeListItem(ListItem):
    """A ListItem that carries a reference to its Recipe."""

    def __init__(self, *args: Any, recipe: Recipe, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.recipe = recipe


class RecipesPane(ListView):
    """Right pane — lists recipes for the selected source."""

    DEFAULT_CSS = """
    .recipe-name {
        color: $success;
        text-style: bold;
        height: auto;
    }
    .recipe-doc {
        color: $text-muted;
        padding-left: 2;
        height: auto;
    }
    .recipe-wrap {
        height: auto;
    }
    ListItem {
        margin-bottom: 0;
    }
    RecipesPane > ListItem.header {
        background: $panel;
        color: $accent;
        text-style: bold;
    }
    RecipesPane > ListItem.header:hover {
        background: $panel;
    }
    RecipesPane > ListItem.grouped {
        padding-left: 2;
    }
    """

    BINDINGS: ClassVar = [
        Binding("enter", "run", "Run", show=True, priority=True),
        Binding("d", "details", "Details"),
    ]

    class RecipeRun(Message):
        def __init__(self, recipe: Recipe) -> None:
            self.recipe = recipe
            super().__init__()

    class RecipeDetails(Message):
        def __init__(self, recipe: Recipe) -> None:
            self.recipe = recipe
            super().__init__()

    def __init__(self) -> None:
        super().__init__(id="recipes")
        self.border_title = "Recipes"
        self._source: Source | None = None
        self._query = ""

    def set_source(self, source: Source) -> None:
        """Replace recipe list when the source changes."""
        self._source = source
        self._rebuild()

    def filter(self, query: str) -> None:
        self._query = query
        self._rebuild()

    def _rebuild(self) -> None:
        self.clear()
        if self._source is None:
            return

        visible = self._source.filter_recipes(self._query)
        groups = group_recipes(visible)
        has_groups = not (len(groups) == 1 and groups[0].name is None)

        for group_name, recipes in groups:
            if has_groups and group_name is not None:
                header = ListItem(Label(f" {group_name} "), classes="header")
                header.disabled = True
                self.append(header)
            for recipe in recipes:
                item = self._build_item(recipe)
                if has_groups and group_name is not None:
                    item.add_class("grouped")
                self.append(item)

        # Skip disabled group headers so the highlight lands on the first selectable recipe.
        self.call_after_refresh(lambda: setattr(self, "index", first_enabled_index(self)))

    def focus_first_enabled(self) -> None:
        """Focus this pane and move the highlight to the first non-disabled item."""
        self.focus()
        if len(self) > 0:
            idx = first_enabled_index(self)
            if self.index is None or (
                self.index != idx and (self.highlighted_child is None or self.highlighted_child.disabled)
            ):
                self.index = idx

    @staticmethod
    def _param_signature(recipe: Recipe) -> str:
        return " ".join(f"<{p.name}>" for p in recipe.parameters)

    def _build_item(self, recipe: Recipe) -> RecipeListItem:
        parts: list[str] = []
        if recipe.parameters:
            parts.append(self._param_signature(recipe))
        if recipe.dependencies:
            parts.append(f"→ {', '.join(recipe.dependencies)}")
        meta = "  ".join(parts)

        name_text = f"{recipe.name} [dim]{meta}[/dim]" if meta else recipe.name

        if recipe.doc:
            content = Vertical(
                Label(name_text, classes="recipe-name", markup=True),
                Label(recipe.doc, classes="recipe-doc"),
                classes="recipe-wrap",
            )
        else:
            content = Label(name_text, classes="recipe-name", markup=True)

        return RecipeListItem(content, recipe=recipe)

    def _highlighted_recipe(self) -> Recipe | None:
        child = self.highlighted_child
        return child.recipe if isinstance(child, RecipeListItem) else None

    def on_list_view_selected(self, event: ListView.Selected) -> None:
        # Handles mouse clicks; keyboard Enter is intercepted by action_run
        event.stop()
        if recipe := self._highlighted_recipe():
            self.post_message(self.RecipeRun(recipe))

    def action_run(self) -> None:
        if recipe := self._highlighted_recipe():
            self.post_message(self.RecipeRun(recipe))

    def action_details(self) -> None:
        if recipe := self._highlighted_recipe():
            self.post_message(self.RecipeDetails(recipe))



from __future__ import annotations

from pathlib import Path
from unittest.mock import patch

from click.testing import CliRunner

from justx.cli.main import main
from tests.utils import run_within_dir


def test_check_just_not_found(tmp_path: Path) -> None:
    runner = CliRunner()
    with patch("justx.cli.commands.check.shutil.which", return_value=None), run_within_dir(tmp_path):
        result = runner.invoke(main, ["check"])
    assert result.exit_code == 1


def test_check_just_found_no_sources(tmp_path: Path) -> None:
    runner = CliRunner()
    with patch("justx.cli.commands.check.shutil.which", return_value="/usr/bin/just"), run_within_dir(tmp_path):
        result = runner.invoke(main, ["check"], env={"JUSTX_HOME": str(tmp_path)})
    assert result.exit_code == 0
    assert "just" in result.output
    assert "(none)" in result.output


def test_check_just_found_with_sources(local_dir: Path, global_dir: Path) -> None:
    runner = CliRunner()
    with run_within_dir(local_dir):
        result = runner.invoke(main, ["check"])
    assert result.exit_code == 0
    assert "simple.just" in result.output  # local source
    assert "justfile" in result.output  # global source



from __future__ import annotations

from dataclasses import dataclass, field
from pathlib import Path

from justx.config import get_justx_home


@dataclass
class DiscoveredPaths:
    global_paths: list[Path] = field(default_factory=list)
    local_paths: list[Path] = field(default_factory=list)


class JustxDiscovery:
    """Discovers justfile paths in global and local scopes."""

    def discover(self, cwd: Path | None = None, justx_home: Path | None = None) -> DiscoveredPaths:
        """Return all justfile paths found in the global and local scopes.

        Args:
            cwd: Local directory to scan. Defaults to the current working directory.
            justx_home: Global home directory. Defaults to $JUSTX_HOME or ~/.justx.
        """
        home = self._resolve_home(justx_home)
        resolved_cwd = cwd or Path.cwd()

        return DiscoveredPaths(
            global_paths=self._discover_global(home),
            local_paths=self._discover_local(resolved_cwd),
        )

    def _resolve_home(self, justx_home: Path | None) -> Path:
        if justx_home is not None:
            return justx_home
        return get_justx_home()

    def _discover_global(self, home: Path) -> list[Path]:
        paths: list[Path] = []
        root = home / "justfile"
        if root.exists():
            paths.append(root)
        paths.extend(self._scan_just_files(home))
        return paths

    def _discover_local(self, cwd: Path) -> list[Path]:
        paths: list[Path] = []
        root = cwd / "justfile"
        if root.exists():
            paths.append(root)
        paths.extend(self._scan_just_files(cwd / ".justx"))
        return paths

    def _scan_just_files(self, directory: Path) -> list[Path]:
        if not directory.is_dir():
            return []
        return sorted([*directory.glob("*.just"), *directory.glob("*.justfile")])



from __future__ import annotations

from pathlib import Path

from justx.justfiles.models import Recipe, Scope
from justx.justfiles.parser import JustfileParser
from justx.justfiles.utils import group_recipes

# ---------------------------------------------------------------------------
# Parser tests
# ---------------------------------------------------------------------------


def test_parse_recipe_with_single_group():
    raw = {
        "name": "build",
        "doc": None,
        "parameters": [],
        "dependencies": [],
        "attributes": [{"group": "dev"}],
    }
    parser = JustfileParser()
    recipe = parser._parse_recipe(raw)
    assert recipe.groups == ["dev"]


def test_parse_recipe_without_group():
    raw = {
        "name": "lint",
        "doc": None,
        "parameters": [],
        "dependencies": [],
        "attributes": [],
    }
    parser = JustfileParser()
    recipe = parser._parse_recipe(raw)
    assert recipe.groups == []


def test_parse_groups_from_justfile(local_dir: Path):
    source = JustfileParser().parse(local_dir / ".justx" / "groups.just", Scope.local)
    recipes = {r.name: r for r in source.recipes}

    assert recipes["build"].groups == ["dev"]
    assert recipes["watch"].groups == ["dev"]
    assert recipes["test"].groups == ["test"]
    assert recipes["lint"].groups == []


# ---------------------------------------------------------------------------
# Grouping logic tests
# ---------------------------------------------------------------------------


def _recipe(name: str, groups: list[str] | None = None) -> Recipe:
    return Recipe(name=name, doc=None, parameters=[], dependencies=[], groups=groups or [])


def test_group_recipes_with_groups():
    recipes = [_recipe("build", ["dev"]), _recipe("test", ["test"])]
    result = group_recipes(recipes)
    assert result == [("dev", [recipes[0]]), ("test", [recipes[1]])]


def test_group_recipes_no_groups():
    recipes = [_recipe("lint"), _recipe("fmt")]
    result = group_recipes(recipes)
    assert result == [(None, recipes)]


def test_group_recipes_mixed():
    build = _recipe("build", ["dev"])
    lint = _recipe("lint")
    result = group_recipes([build, lint])
    assert result == [(None, [lint]), ("dev", [build])]


def test_group_recipes_preserves_order_within_group():
    build = _recipe("build", ["dev"])
    watch = _recipe("watch", ["dev"])
    result = group_recipes([build, watch])
    assert result == [("dev", [build, watch])]


def test_group_recipes_empty_list():
    assert group_recipes([]) == []


# ---------------------------------------------------------------------------
# Integration test (parser + model)
# ---------------------------------------------------------------------------


def test_parsed_source_contains_groups(local_dir: Path):
    source = JustfileParser().parse(local_dir / ".justx" / "groups.just", Scope.local)
    recipes = {r.name: r for r in source.recipes}

    assert set(recipes) == {"build", "watch", "test", "lint"}
    assert recipes["build"].groups == ["dev"]
    assert recipes["watch"].groups == ["dev"]
    assert recipes["test"].groups == ["test"]
    assert recipes["lint"].groups == []



from __future__ import annotations

import shutil
import sys

import click
from rich.console import Console
from rich.markup import escape

from justx.justfiles.discovery import JustxDiscovery


@click.command("check")
def check_cmd() -> None:
    """Verify that just is installed and show discovered justfile locations."""
    console = Console()
    stderr = Console(stderr=True)

    just_bin = shutil.which("just")
    if just_bin is None:
        stderr.print("[bold red]ERROR:[/bold red] just binary not found on PATH.")
        stderr.print("Install it from https://github.com/casey/just")
        sys.exit(1)

    console.print(f"[bold]just:[/bold] [cyan]{escape(just_bin)}[/cyan]")

    paths = JustxDiscovery().discover()

    console.print("\n[bold]Global justfiles:[/bold]")
    if paths.global_paths:
        for path in paths.global_paths:
            console.print(f"  {escape(str(path))}")
    else:
        console.print("  (none)")

    console.print("\n[bold]Local justfiles:[/bold]")
    if paths.local_paths:
        for path in paths.local_paths:
            console.print(f"  {escape(str(path))}")
    else:
        console.print("  (none)")



from __future__ import annotations

import urllib.error
from pathlib import Path
from unittest.mock import patch

import pytest
from click.testing import CliRunner

from justx.cli.main import main

FAKE_FILES = [
    {"name": "docker.just", "download_url": "https://example.com/docker.just"},
    {"name": "git.just", "download_url": "https://example.com/git.just"},
    {"name": "uv.just", "download_url": "https://example.com/uv.just"},
]

runner = CliRunner()


# ---------------------------------------------------------------------------
# justx init (no flag)
# ---------------------------------------------------------------------------


def test_init_creates_user_just(tmp_path: Path) -> None:
    result = runner.invoke(main, ["init"], input="y\n", env={"JUSTX_HOME": str(tmp_path)})
    assert result.exit_code == 0
    user_just = tmp_path / "user.just"
    assert user_just.exists()
    assert "greet" in user_just.read_text()


def test_init_aborted(tmp_path: Path) -> None:
    result = runner.invoke(main, ["init"], input="n\n", env={"JUSTX_HOME": str(tmp_path)})
    assert result.exit_code == 0
    assert "Aborted" in result.output
    assert not (tmp_path / "user.just").exists()


def test_init_skips_existing_user_just(tmp_path: Path) -> None:
    (tmp_path / "user.just").write_text("# existing\n")
    result = runner.invoke(main, ["init"], input="y\n", env={"JUSTX_HOME": str(tmp_path)})
    assert result.exit_code == 0
    assert "already exists" in result.output
    assert (tmp_path / "user.just").read_text() == "# existing\n"


# ---------------------------------------------------------------------------
# justx init --download-examples
# ---------------------------------------------------------------------------


@pytest.fixture
def mock_fetch():
    with patch("justx.cli.commands.init._fetch_example_files", return_value=FAKE_FILES):
        yield


def test_download_examples_all_selected(tmp_path: Path, mock_fetch: None) -> None:
    with (
        patch("justx.cli.commands.init.urllib.request.urlretrieve") as mock_retrieve,
        patch("justx.cli.commands.init.questionary.checkbox") as mock_cb,
    ):
        mock_cb.return_value.ask.return_value = FAKE_FILES
        result = runner.invoke(main, ["init", "--download-examples"], input="y\n", env={"JUSTX_HOME": str(tmp_path)})
    assert result.exit_code == 0
    assert mock_retrieve.call_count == 3
    for f in FAKE_FILES:
        mock_retrieve.assert_any_call(f["download_url"], tmp_path / f["name"])
    assert not (tmp_path / "user.just").exists()


def test_download_examples_partial_selection(tmp_path: Path, mock_fetch: None) -> None:
    selected = [FAKE_FILES[0], FAKE_FILES[2]]  # docker.just and uv.just
    with (
        patch("justx.cli.commands.init.urllib.request.urlretrieve") as mock_retrieve,
        patch("justx.cli.commands.init.questionary.checkbox") as mock_cb,
    ):
        mock_cb.return_value.ask.return_value = selected
        result = runner.invoke(main, ["init", "--download-examples"], input="y\n", env={"JUSTX_HOME": str(tmp_path)})
    assert result.exit_code == 0
    assert mock_retrieve.call_count == 2


def test_download_examples_none_selected(tmp_path: Path, mock_fetch: None) -> None:
    with patch("justx.cli.commands.init.questionary.checkbox") as mock_cb:
        mock_cb.return_value.ask.return_value = []
        result = runner.invoke(main, ["init", "--download-examples"], input="y\n", env={"JUSTX_HOME": str(tmp_path)})
    assert result.exit_code == 0
    assert "No files selected" in result.output


def test_download_examples_skips_existing_file(tmp_path: Path, mock_fetch: None) -> None:
    (tmp_path / "docker.just").write_text("# existing\n")
    with (
        patch("justx.cli.commands.init.urllib.request.urlretrieve") as mock_retrieve,
        patch("justx.cli.commands.init.questionary.checkbox") as mock_cb,
    ):
        mock_cb.return_value.ask.return_value = FAKE_FILES
        result = runner.invoke(main, ["init", "--download-examples"], input="y\n", env={"JUSTX_HOME": str(tmp_path)})
    assert result.exit_code == 0
    assert "already exists" in result.output
    assert mock_retrieve.call_count == 2  # git.just and uv.just only


def test_download_examples_aborted(tmp_path: Path) -> None:
    with patch("justx.cli.commands.init._fetch_example_files") as mock_fetch_fn:
        result = runner.invoke(main, ["init", "--download-examples"], input="n\n", env={"JUSTX_HOME": str(tmp_path)})
    assert result.exit_code == 0
    assert "Aborted" in result.output
    mock_fetch_fn.assert_not_called()


def test_download_examples_network_error(tmp_path: Path) -> None:
    with patch(
        "justx.cli.commands.init._fetch_example_files",
        side_effect=urllib.error.URLError("connection refused"),
    ):
        result = runner.invoke(main, ["init", "--download-examples"], input="y\n", env={"JUSTX_HOME": str(tmp_path)})
    assert result.exit_code == 1
    assert "Could not fetch examples" in result.output


def test_download_examples_no_files_found(tmp_path: Path) -> None:
    with (
        patch("justx.cli.commands.init._fetch_example_files", return_value=[]),
        patch("justx.cli.commands.init.questionary.checkbox") as mock_cb,
    ):
        result = runner.invoke(main, ["init", "--download-examples"], input="y\n", env={"JUSTX_HOME": str(tmp_path)})
    assert result.exit_code == 0
    assert "No example files found" in result.output
    mock_cb.assert_not_called()



from __future__ import annotations

import os
from pathlib import Path

DEFAULT_JUSTX_HOME = Path.home() / ".justx"


def get_justx_home() -> Path:
    """Resolve the justx home directory: $JUSTX_HOME env var, or ~/.justx."""
    env = os.environ.get("JUSTX_HOME")
    if env:
        return Path(env)
    return DEFAULT_JUSTX_HOME



from __future__ import annotations

import json
import re
import shutil
import subprocess
from pathlib import Path

from justx.justfiles.exceptions import JustInvocationError, JustNotFoundError
from justx.justfiles.models import Parameter, ParameterKind, Recipe, RecipeDefault, Scope, Source, WorkingDirMode


class JustfileParser:
    """Parses justfiles into Source models by invoking just."""

    def parse(self, path: Path, scope: Scope) -> Source:
        """Parse a justfile and return a Source.

        Raises:
            FileNotFoundError: if the justfile does not exist.
            JustNotFoundError: if the just binary is not on PATH.
            JustInvocationError: if just exits with a non-zero status.
        """
        if not path.exists():
            raise FileNotFoundError(f"Justfile not found: {path}")  # noqa: TRY003

        binary = self._require_just()
        data = self._dump(binary, path)
        recipes = [self._parse_recipe(r) for r in data.get("recipes", {}).values()]
        working_dir_mode = self._parse_working_dir_mode(path)

        return Source(name=path.stem, scope=scope, path=path, recipes=recipes, working_dir_mode=working_dir_mode)

    def _parse_working_dir_mode(self, path: Path) -> WorkingDirMode:
        _DIRECTIVE_RE = re.compile(r"#\s*justx:\s*working-directory\s*=\s*(\S+)")

        text = path.read_text()
        m = _DIRECTIVE_RE.search(text)
        if m and m.group(1) == "justfile":
            return WorkingDirMode.justfile
        return WorkingDirMode.cwd

    def _require_just(self) -> str:
        binary = shutil.which("just")
        if binary is None:
            raise JustNotFoundError
        return binary

    def _dump(self, binary: str, path: Path) -> dict:
        result = subprocess.run(
            [binary, "--dump", "--dump-format", "json", "--justfile", str(path)],
            capture_output=True,
            text=True,
        )
        if result.returncode != 0:
            raise JustInvocationError(result.returncode, result.stderr.strip())
        return json.loads(result.stdout)

    def _parse_recipe(self, raw: dict) -> Recipe:
        parameters = [self._parse_parameter(p) for p in raw.get("parameters", [])]
        dependencies = [dep["recipe"] for dep in raw.get("dependencies", [])]
        groups = [attr["group"] for attr in raw.get("attributes", []) if "group" in attr]
        return Recipe(
            name=raw["name"],
            doc=raw.get("doc"),
            parameters=parameters,
            dependencies=dependencies,
            groups=groups,
        )

    def _parse_parameter(self, raw: dict) -> Parameter:
        raw_default = raw.get("default")
        if raw_default is None:
            default = None
            has_default = False
        else:
            default = RecipeDefault(
                value=raw_default,
                expression=not isinstance(raw_default, str),
            )
            has_default = True
        kind = self._parameter_kind(raw["kind"], has_default)
        return Parameter(name=raw["name"], default=default, kind=kind)

    @staticmethod
    def _parameter_kind(just_kind: str, has_default: bool) -> ParameterKind:
        if just_kind in ("star", "plus"):
            return ParameterKind.variadic
        if has_default:
            return ParameterKind.optional
        return ParameterKind.required



from justx.justfiles.models import Recipe, RecipeGroup


def group_recipes(recipes: list[Recipe]) -> list[RecipeGroup]:
    """Group recipes by group name, alphabetically, ungrouped last.

    Returns a list of RecipeGroup tuples. If no recipes have groups,
    returns [RecipeGroup(None, recipes)] for flat-list behavior.
    """
    if not recipes:
        return []

    if not any(r.groups for r in recipes):
        return [RecipeGroup(None, recipes)]

    seen_groups: dict[str, list[Recipe]] = {}
    ungrouped: list[Recipe] = []

    for recipe in recipes:
        if recipe.groups:
            for group in recipe.groups:
                seen_groups.setdefault(group, []).append(recipe)
        else:
            ungrouped.append(recipe)

    result: list[RecipeGroup] = []
    if ungrouped:
        result.append(RecipeGroup(None, ungrouped))
    result.extend(RecipeGroup(name, seen_groups[name]) for name in sorted(seen_groups))
    return result



from __future__ import annotations

from typing import ClassVar

from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import Vertical
from textual.screen import Screen
from textual.widgets import Footer, Label

from justx.justfiles.models import ParameterKind, Recipe, Source


class RecipeDetailScreen(Screen[None]):
    CSS = """
    RecipeDetailScreen {
        align: center middle;
    }
    #dialog {
        width: 80;
        height: auto;
        border: solid dodgerblue;
        border-title-color: dodgerblue;
        padding: 1 2;
        background: $surface;
    }
    #name {
        text-style: bold;
        color: $success;
        margin-bottom: 1;
    }
    .section-header {
        text-style: bold;
        color: $accent;
        margin-top: 1;
    }
    .detail-value {
        color: $text;
        padding-left: 2;
    }
    .detail-muted {
        color: $text-muted;
        padding-left: 2;
    }
.param-row {
        height: auto;
        padding-left: 2;
    }
    """
    BINDINGS: ClassVar = [
        Binding("left", "dismiss", "Back", show=True, priority=True),
        Binding("backspace", "dismiss", "Back", show=False, priority=True),
    ]

    def __init__(self, recipe: Recipe, source: Source | None = None) -> None:
        super().__init__()
        self._recipe = recipe
        self._source = source

    def compose(self) -> ComposeResult:
        with Vertical(id="dialog") as dialog:
            dialog.border_title = "Recipe Details"
            yield Label(self._recipe.name, id="name")

            if self._recipe.doc:
                yield Label("Description", classes="section-header")
                yield Label(self._recipe.doc, classes="detail-value")

            if self._source is not None:
                yield Label("Justfile", classes="section-header")
                yield Label(str(self._source.path), classes="detail-muted")

            if self._recipe.dependencies:
                yield Label("Dependencies", classes="section-header")
                yield Label(", ".join(self._recipe.dependencies), classes="detail-value")

            if self._recipe.parameters:
                yield Label("Parameters", classes="section-header")
                for param in self._recipe.parameters:
                    yield self._build_param_label(param)
            else:
                yield Label("Parameters", classes="section-header")
                yield Label("none", classes="detail-muted")

        yield Footer()

    def _build_param_label(self, param) -> Label:
        kind_map = {
            ParameterKind.required: "required",
            ParameterKind.optional: "optional",
            ParameterKind.variadic: "variadic",
        }
        if param.default is None:
            default = ""
        elif param.default.expression:
            default = ", default: <expression>"
        else:
            default = f", default: {param.default.value!r}"
        return Label(f"  {param.name}  ({kind_map[param.kind]}{default})", classes="param-row")

    def action_dismiss(self) -> None:
        self.dismiss()



from __future__ import annotations

from typing import ClassVar

from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import Horizontal, Vertical
from textual.screen import Screen
from textual.widgets import Button, Footer, Input, Label

from justx.justfiles.models import Parameter, ParameterKind, Recipe, Source


class RecipeScreen(Screen[list[str] | None]):
    CSS = """
    RecipeScreen {
        align: center middle;
    }
    #dialog {
        width: 70;
        height: auto;
        border: solid dodgerblue;
        border-title-color: dodgerblue;
        padding: 1 2;
        background: $surface;
    }
    #title {
        text-style: bold;
        color: $success;
        margin-bottom: 1;
    }
    #doc {
        color: $text-muted;
        margin-bottom: 1;
    }
    #expression-notice {
        color: $text-muted;
        margin-bottom: 1;
    }
    .param-row {
        height: auto;
        margin-bottom: 1;
    }
    .param-label {
        width: 1fr;
        padding-top: 1;
    }
    .param-input {
        width: 2fr;
    }
    #buttons {
        height: auto;
        margin-top: 1;
        align: right middle;
    }
    """
    BINDINGS: ClassVar = [
        Binding("escape", "cancel", "Cancel", show=True, priority=True),
        Binding("ctrl+enter", "run", "Run", show=True, priority=True),
        Binding("d", "details", "Details", show=True, priority=True),
        Binding("up", "focus_prev_field", "↑ Prev", show=True, priority=True),
        Binding("down", "focus_next_field", "↓ Next", show=True, priority=True),
        Binding("left", "focus_button_left", show=False, priority=True),
        Binding("right", "focus_button_right", show=False, priority=True),
    ]

    def __init__(self, recipe: Recipe, source: Source | None = None) -> None:
        super().__init__()
        self._recipe = recipe
        self._source = source

    def compose(self) -> ComposeResult:
        with Vertical(id="dialog") as dialog:
            dialog.border_title = "Run Recipe"
            yield Label(self._recipe.name, id="title")
            if self._recipe.doc:
                yield Label(self._recipe.doc, id="doc")
            if any(p.default and p.default.expression for p in self._recipe.parameters):
                yield Label(
                    "Expression defaults: fill from the top, or leave all blank.",
                    id="expression-notice",
                )
            for param in self._recipe.parameters:
                yield self._build_param_row(param)
            with Horizontal(id="buttons"):
                yield Button("Cancel", variant="default", id="cancel")
                yield Button("Run", variant="primary", id="run")
        yield Footer()

    def _build_param_row(self, param: Parameter) -> Horizontal:
        suffix = (
            " *" if param.kind == ParameterKind.required else (" ..." if param.kind == ParameterKind.variadic else "")
        )
        if param.default is None or param.default.expression:
            input_value = ""
            input_placeholder = "<expression>" if (param.default and param.default.expression) else ""
        else:
            input_value = param.default.value or ""
            input_placeholder = ""
        return Horizontal(
            Label(f"{param.name}{suffix}", classes="param-label"),
            Input(
                value=input_value,
                placeholder=input_placeholder,
                id=f"param-{param.name}",
                classes="param-input",
            ),
            classes="param-row",
        )

    def _collect_args(self) -> list[str]:
        """Collect positional arguments to pass to just.

        just arguments are purely positional, so we can only pass a prefix of
        them — once we stop, all remaining params use their just-side defaults
        (including expression defaults like ``(arch + "-unknown-unknown")``).

        Trailing empty inputs are therefore stripped: an empty field at the end
        means "use just's default", not "pass an empty string".  An empty field
        that precedes a filled one is unavoidable and will be passed as ``""``
        (overriding that param's default with an empty string) — this is a known
        limitation of just's positional calling convention.
        """
        values = [self.query_one(f"#param-{p.name}", Input).value for p in self._recipe.parameters]
        while values and values[-1] == "":
            values.pop()
        return values

    def on_mount(self) -> None:
        self._update_field_states()

    def on_input_changed(self, event: Input.Changed) -> None:
        self._update_field_states()

    def _update_field_states(self) -> None:
        """Disable fields that follow an empty expression-default field.

        Because just arguments are positional, filling param N while leaving
        param M (M < N, expression default) empty would pass "" for M — not
        its expression default. Disabling downstream fields prevents that
        impossible state from being created in the first place.
        """
        blocked = False
        for param in self._recipe.parameters:
            widget = self.query_one(f"#param-{param.name}", Input)
            if blocked:
                widget.disabled = True
            else:
                widget.disabled = False
                if param.default and param.default.expression and not widget.value:
                    blocked = True

    def _inputs(self) -> list[Input]:
        return [i for i in self.query(Input) if not i.disabled]

    def _advance_focus(self, current: Input) -> None:
        inputs = self._inputs()
        idx = inputs.index(current)
        if idx < len(inputs) - 1:
            inputs[idx + 1].focus()
        else:
            self.query_one("#run", Button).focus()

    def _retreat_focus(self, current: Input) -> None:
        inputs = self._inputs()
        idx = inputs.index(current)
        if idx > 0:
            inputs[idx - 1].focus()

    def on_input_submitted(self, event: Input.Submitted) -> None:
        self._advance_focus(event.input)

    def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "run":
            self.dismiss(self._collect_args())
        elif event.button.id == "cancel":
            self.dismiss(None)

    def action_details(self) -> None:
        from justx.tui.screens.recipe_detail import RecipeDetailScreen

        self.app.push_screen(RecipeDetailScreen(self._recipe, self._source))

    def action_cancel(self) -> None:
        self.dismiss(None)

    def action_run(self) -> None:
        self.dismiss(self._collect_args())

    def action_focus_next_field(self) -> None:
        if isinstance(self.focused, Input):
            self._advance_focus(self.focused)

    def action_focus_button_left(self) -> None:
        if isinstance(self.focused, Button) and self.focused.id == "run":
            self.query_one("#cancel", Button).focus()

    def action_focus_button_right(self) -> None:
        if isinstance(self.focused, Button) and self.focused.id == "cancel":
            self.query_one("#run", Button).focus()

    def action_focus_prev_field(self) -> None:
        if isinstance(self.focused, Input):
            self._retreat_focus(self.focused)
        elif isinstance(self.focused, Button) and self.focused.id == "run":
            inputs = self._inputs()
            if inputs:
                inputs[-1].focus()
